import pysam
from igv_reports.featureTree import FeatureTree
from igv_reports.chralias import get_alias
from igv_reports.stream import getstream

class Feature:

    def __init__(self, chr, start, end, text, name='', chr2=None, start2=None, end2=None):
        self.chr = chr
        self.start = start
        self.end = end
        self.chr2 = chr2
        self.start2 = start2
        self.end2 = end2
        self.text = text
        self.name = name

'''
Return an appropriate reader for the given path
'''
def get_featurereader(path):

    if path.endswith(".gz"):
        # Might be a tabix file
        try:
            return TabixFeatureReader(pysam.TabixFile(path))
        except:
            return FeatureReader(path)

    else:
        return FeatureReader(path)


'''
Tabix indexed file reader
'''

class TabixFeatureReader:

    def __init__(self, tabix):
        self.file = tabix
        self.aliastable = {}

    def slice(self, region=None, region2=None):

        tb = self.file
        if region is not None:
            chr = self.get_chrname(region["chr"])
            it = self._fetch(chr, region["start"], region["end"])
        else:
            it = tb.fetch()

        data = ""
        for row in it:
            data += row + '\n'

        if region2 is not None:
            chr = self.get_chrname(region2["chr"])
            it = self._fetch(chr, region2["start"], region2["end"])
            for row in it:
                data += row + '\n'

        return data

    def _fetch(self, chr, start, end):

        if chr is None:
            return []

        tb = self.file
        try:
            range_string = f'{chr}:{start}-{end}'
            return tb.fetch(range_string)

        except ValueError:
            # Possible chr alias error (e.g. 1 vs chr1)
            try:
                alias = get_alias(chr)
                range_string = f'{alias}:{start}-{end}'
                value = tb.fetch(range_string)
                self.aliastable[chr] = alias
                return value
            except ValueError:
                self.aliastable[chr] = None
                print(f'WARNING: No data with sequence name {chr} found in file {self.file.filename}')
                return []

    def get_chrname(self, c):
        if c in self.aliastable:
            return self.aliastable[c]
        else:
            return c


'''
Non-indexed file reader.  Emulates a tabix style query interface
'''

class FeatureReader:

    def __init__(self, file):

        self.file = file
        self.tree = None
        self.aliastable = {}

    def slice(self, region=None, region2=None):

        if region == None:
            f = None
            try:
                f = getstream(self.file)
                return f.read()
            finally:
                if f:
                    f.close()

        else:

            # If first time through read the entire file and create a queyable feature tree.
            if not self.tree:
                features = parse(self.file)
                for f in features:
                    if f.chr not in self.aliastable:
                        self.aliastable[get_alias(f.chr)] = f.chr
                self.tree = FeatureTree(features)

            features = []

            reference = self.get_chrname(region["chr"])
            start = region["start"]
            end = region["end"]
            feature_intervals = self.tree.query(reference, start, end)
            if feature_intervals:
                for i in feature_intervals:
                    features.append(i.data)

            if region2 is not None:
                reference = self.get_chrname(region2["chr"])
                start = region2["start"]
                end = region2["end"]
                feature_intervals = self.tree.query(reference, start, end)
                if feature_intervals:
                    for i in feature_intervals:
                        features.append(i.data)

        content = ''
        for f in features:
            content += f.text
        return content

    def get_chrname(self, c):
        if c in self.aliastable:
            return self.aliastable[c]
        else:
            return c


'''
A "mock reader" initialized with a list of bed features
'''

class MockReader:

    def __init__(self, features):
        self.features = features
        self.tree = FeatureTree(features)
        self.aliastable = {}
        for f in features:
            if f.chr not in self.aliastable:
                self.aliastable[get_alias(f.chr)] = f.chr

    def slice(self, region=None, region2=None):

        if region == None:
            return self.features

        else:

            features = []

            reference = self.get_chrname(region["chr"])
            start = region["start"]
            end = region["end"]
            feature_intervals = self.tree.query(reference, start, end)

            if feature_intervals:
                for i in feature_intervals:
                    features.append(i.data)

            if region2 is not None:
                reference = self.get_chrname(region2["chr"])
                start = region2["start"]
                end = region2["end"]
                feature_intervals = self.tree.query(reference, start, end)

                if feature_intervals:
                    for i in feature_intervals:
                        features.append(i.data)

            content = ''
            for f in features:
                chr = f.chr
                content += f"{chr}\t{f.start}\t{f.end}\n"
            return content

    def get_chrname(self, c):
        if c in self.aliastable:
            return self.aliastable[c]
        else:
            return c

def parse(path, format=None):
    '''
    Parse a feature file and return an array of feature objects.  Supported formats are bed, gff, and gtf.
    :param path: Path to feature file, which can be local or url
    :param format: File format, bed | gtf | gff
    :param split_bool: Boolean specifying whether view is multi locus or not
    :return: List of feature objects {chr, start, end, text, name}
    '''

    f = None
    try:
        f = getstream(path)
        if not format:
            format = infer_format(path)
        if format == 'bed':
            return parse_bed(f)
        elif format == 'gff'  or format == 'gff3' or format == 'gtf':
            return parse_gff(f)
        elif format == 'tab':
            return parse_tab(f)
        elif format == 'bedpe':
            return parse_bedpe(f)
        elif format == 'refgene':
            return parse_refgene(f)
        else:
            raise Exception("Unknown file format: " + path)
    finally:
        if f:
            f.close()


def parse_bed(f):
    features = []
    for line in f:
        if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
            tokens = line.rstrip('\n').rstrip('\r').split('\t')
            if len(tokens) >= 3:
                chr = tokens[0]
                start = int(tokens[1])
                end = int(tokens[2])
                name = tokens[3] if len(tokens) > 3 else ''
                features.append(Feature(chr, start, end, line, name))
    return features


def parse_refgene(f):
    features = []
    for line in f:
        if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
            tokens = line.rstrip('\n').rstrip('\r').split('\t')
            if len(tokens) >= 3:
                chr = tokens[2]
                start = int(tokens[4])
                end = int(tokens[5])
                name = tokens[12] if len(tokens) > 12 else ''
                features.append(Feature(chr, start, end, line, name))
    return features


def parse_bedpe(f):
    features = []
    for line in f:
        if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
            tokens = line.rstrip('\n').rstrip('\r').split('\t')
            if len(tokens) >= 6:
                chr = tokens[0]
                start = int(tokens[1])
                end = int(tokens[2])
                chr2 = tokens[3]
                start2 = int(tokens[4])
                end2 = int(tokens[5])
                name = tokens[6] if len(tokens) > 6 else ''
                features.append(Feature(chr, start, end, line, name, chr2, start2, end2))
    return features


def parse_gff(f):
    features = []
    for line in f:
        if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
            tokens = line.rstrip('\n').rstrip('\r').split('\t')
            # if we encounter a blank or malformed line (no start and end coords), skip it
            if len(tokens) < 5:
                continue
            chr = tokens[0]
            start = int(tokens[3]) - 1
            end = int(tokens[4])
            name = ''
            features.append(Feature(chr, start, end, line, name))

    return features


def parse_tab(f):
    rows = []
    for line in f:
        if not (line.startswith('#') or line.startswith('track') or line.startswith('browser')):
            tokens = line.rstrip('\n').rstrip('\r').split('\t')
            if len(tokens) > 2:
                rows.append(tokens)
    return rows


def infer_format(filename):
    '''
    Infer the genomic file format from the filename.  First known formats are checked.  Next presenece of
    the magic string "refgene" in the filename is checked for UCSC refgene files.  This is a legacy
    IGV convention.  The order is important, a recognized extension wins.
    NOTE: Formats are for output data uris.  CRAM format is converted to BAM before output.
    :param filename:
    :return:
    '''
    filename = filename.lower()
    if (filename.endswith(".gz")):
        filename = filename[:-3]

    if filename.endswith(".bam"):
        return "bam"
    if filename.endswith(".cram"):
        return "cram"
    elif filename.endswith(".vcf"):
        return "vcf"
    elif filename.endswith(".bed"):
        return "bed"
    elif filename.endswith(".bedpe"):
        return "bedpe"
    elif filename.endswith(".gff") or filename.endswith(".gff3"):
        return "gff"
    elif filename.endswith(".gtf"):
        return "gtf"
    elif filename.find("refgene"):
        return "refgene"
    else:
        idx = filename.rfind(".")
        if idx > 0:
            return filename[idx + 1:]
        else:
            return None
        idx = filename.rfind(".")
        if idx > 0:
            return filename[idx + 1:]
        else:
            return None



import os
import sys
import json
import math
import argparse
from urllib.request import urlopen
from igv_reports import datauri, tracks, utils
from igv_reports.varianttable import VariantTable
from igv_reports.bedtable import BedTable
from igv_reports.bedtable import BedpeTable
from igv_reports.bedtable import JunctionBedTable
from igv_reports.generictable import GenericTable
from igv_reports.regions import parse_region
from igv_reports.feature import MockReader
from igv_reports.fasta import FastaReader
from igv_reports.ideogram import IdeogramReader
from igv_reports.genome import get_genome

'''
Create an html report.  This is the main function for the application.
'''
def create_report(args):

    trackconfigs = []
    trackreaders = []
    session_dict = {}

    # Read the variant data
    variants_file = args.sites

    if variants_file.endswith(".bcf") or variants_file.endswith(".vcf") or variants_file.endswith(".vcf.gz"):
        table = VariantTable(variants_file, args.info_columns, args.info_columns_prefixes, args.samples,
                             args.sample_columns, args.idlink)

    elif variants_file.endswith(".bed") or variants_file.endswith(".bed.gz"):
        if args.type is not None and args.type == "junction":
            table = JunctionBedTable(variants_file, args.info_columns)
        else:
            table = BedTable(variants_file)

    elif variants_file.endswith(".bedpe") or variants_file.endswith(".bedpe.gz"):
        table = BedpeTable(variants_file)

    elif variants_file.endswith(".maf") or variants_file.endswith(".maf.gz") or (
            args.sequence is not None and args.begin is not None and args.end is not None):
        # A hack -- since these file formats are not supported by igv.js mock up a bed style track from a table of generic features
        table = GenericTable(variants_file, args.info_columns, args.sequence, args.begin, args.end, args.zero_based)
        flist = []
        for tuple in table.features:
            flist.append(tuple[0])
        trackconfigs.append({
            "config": {"name": "variants", "type": "annotation", "format": "bed"},
            "reader": MockReader(flist)
        })

    # Track json array.  Tracks can come from (1) tracks CL argument, (2) genome CL argument, and (3) track_config Cl argument
    trackjson = []

    # Check for optional genome argument.  If supplied an igv.js genome json definition is used in lieu of a fasta file
    if args.genome is not None:
        genome = get_genome(args.genome)
        if args.fasta is None:
            args.fasta = genome["fastaURL"]
        if args.ideogram is None and "cytobandURL" in genome:
            args.ideogram = genome["cytobandURL"]
        if "tracks" in genome:
            for config in genome["tracks"]:
                trackjson.append(config)

    if args.tracks is not None:
        for track in args.tracks:
           trackjson.append(tracks.get_track_json_dict(track))

    if args.track_config is not None:
        for trackobj in args.track_config:
            with open(trackobj) as f:
                j = json.load(f)
                for c in j:
                    trackjson.append(c)

    # Create file readers for tracks.  This is done outside the locus loop so initialization happens once

    for config in trackjson:
        reader = utils.getreader(config, None, args.fasta)
        trackconfigs.append({
            "config": config,
            "reader": reader
        })



    # Other readers
    fasta_reader = FastaReader(args.fasta)
    if (args.ideogram):
        ideogram_reader = IdeogramReader(args.ideogram)
    else:
        ideogram_reader = None

    # loop through regions defined from variant, annotation, or bedpe files,  creating an igv.js session for each one
    flanking = 0
    if args.flanking is not None:
        flanking = float(args.flanking)

    i = 0
    for tuple in table.features:
        i += 1
        print(f"Working on variant {i}/{len(table.features)}")

        feature = tuple[0]
        unique_id = tuple[1]

        # If a variant feature (=> row in the table) has an explicit session id use it, otherwise use the row id (i.e. unique_id)
        if hasattr(feature, "session_id"):
            session_id = feature.session_id
        else:
            session_id = str(unique_id)

        if session_id not in session_dict:

            # Placeholder variable for possible second region (bedpe files)
            region2 = None

            # Define a genomic region around the variant
            if hasattr(feature, "viewport"):
                region = parse_region(feature.viewport)
                chr = region["chr"]
                start = region["start"]
                end = region["end"]
            else:
                chr = feature.chr
                start = int(math.floor(feature.start - flanking / 2))
                start = max(start, 1)  # bound start to 1
                end = int(math.ceil(feature.end + flanking / 2))
                region = {
                    "chr": chr,
                    "start": start,
                    "end": end
                }

                # If feature has a second locus (bedpe file) create the region here.
                if hasattr(feature, 'chr2') and feature.chr2 is not None:
                    chr2 = feature.chr2
                    start2 = int(math.floor(feature.start2 - flanking / 2))
                    start2 = max(start2, 1)  # bound start to 1
                    end2 = int(math.ceil(feature.end2 + flanking / 2))
                    region2 = {
                        "chr": chr2,
                        "start": start2,
                        "end": end2
                    }

            # Fasta
            data = fasta_reader.slice(region)
            fa = '>' + chr + ':' + str(start) + '-' + str(end) + '\n' + data

            if region2 is not None:
                data2 = fasta_reader.slice(region2)
                fa += '\n' + '>' + chr2 + ':' + str(start2) + '-' + str(end2) + '\n' + data2

            fasta_uri = datauri.get_data_uri(fa)
            fastaJson = {
                "fastaURL": fasta_uri,
            }

            # Ideogram
            if (args.ideogram):
                ideo_string = ideogram_reader.get_data(region["chr"])
                if region2 is not None:
                    ideo_string += ideogram_reader.get_data(region2["chr"])
                ideo_uri = datauri.get_data_uri(ideo_string)
                fastaJson["cytobandURL"] = ideo_uri

            # Initial locus
            if (hasattr(feature, "viewport")):
                initial_locus = feature.viewport
            else:
                initial_locus = locus_string(feature.chr, feature.start, feature.end)
                if region2 is not None:
                    initial_locus += f' {locus_string(feature.chr2, feature.start2, feature.end2)}'

            session_json = {
                "locus": initial_locus,
                "reference": fastaJson,
                "tracks": []
            }

            track_objects = []
            # Loop through user supplied track configs
            # "cram" input format is converted to "bam" for output track configs
            for tc in trackconfigs:
                trackobj = tc["config"];

                # Set some defaults if not specified
                if "url" in trackobj:
                    default_trackobj = tracks.get_track_json_dict(trackobj["url"]);
                    if "type" not in trackobj:
                        trackobj["type"] = default_trackobj["type"]
                    if "format" not in trackobj:
                        trackobj["format"] = default_trackobj["format"]
                    if trackobj["format"] == "cram":
                        trackobj["format"] = "bam"
                    if "name" not in trackobj:
                        trackobj["name"] = default_trackobj["url"]

                # Indexes are not used with data URIs
                if "indexURL" in trackobj:
                    del trackobj["indexURL"]

                reader = tc["reader"]
                data = reader.slice(region, region2)
                trackobj["url"] = datauri.get_data_uri(data)
                track_objects.append(trackobj)

            track_order = 1
            for trackobj in track_objects:
                if (trackobj["type"] == "alignment"):
                    trackobj["height"] = 500
                    is_snv = feature.end - feature.start == 1
                    if (trackobj["type"]) == "alignment" and (args.sort is not None or is_snv) and (
                            args.sort != 'NONE'):
                        sort_option = 'BASE' if args.sort is None else args.sort.upper()
                        trackobj["sort"] = {
                            "option": sort_option,
                            "chr": chr,
                            "position": str(feature.start + 1),
                            "direction": "ASC"
                        }
                if "order" not in trackobj:
                    trackobj["order"] = track_order
                session_json["tracks"].append(trackobj)
                track_order += 1

            # Build the session data URI

            session_string = json.dumps(session_json)
            session_uri = datauri.get_data_uri(session_string)
            session_dict[session_id] = session_uri

    session_dict = json.dumps(session_dict)

    template_file = args.template
    if None == template_file:
        if 'junction' == args.type:
            template_file = os.path.dirname(sys.modules['igv_reports'].__file__) + '/templates/junction_template.html'
        else:
            template_file = os.path.dirname(sys.modules['igv_reports'].__file__) + '/templates/variant_template-2.html'

    output_file = args.output

    standalone = args.standalone
    with open(template_file, "r") as f:
        data = f.readlines()

        with open(output_file, "w") as o:

            for i, line in enumerate(data):

                if args.title is not None and line.startswith("<!--title-->"):
                    o.write("<h1>" + args.title + "</h1>")

                if standalone:
                    if line.strip().startswith("<script") and ".js\"" in line:
                        inline_script(line, o, "js")
                        continue
                    elif line.strip().startswith("<link") and line.strip().endswith("css\">"):
                        inline_script(line, o, "css")
                        continue
                j = line.find('"@TABLE_JSON@"')
                if j >= 0:
                    line = line.replace('"@TABLE_JSON@"', table.to_JSON())

                j = line.find('"@SESSION_DICTIONARY@"')
                if j >= 0:
                    line = line.replace('"@SESSION_DICTIONARY@"', session_dict)

                o.write(line)


def inline_script(line, o, source_type):
    # <script type="text/javascript" src="https://igv.org/web/test/dist/igv.min.js"></script>
    if source_type == "js":
        s = line.find('src="')
        offset = 5
        o.write('<script type="text/javascript">\n')
    elif source_type == "css":
        s = line.find('href="')
        offset = 6
        o.write('<style type="text/css">\n')
    else:
        raise KeyError("Inline script must be either js- or css-file")
    if s > 0:
        e = line.find('">', s)
        url = line[s + offset:e]
        response = urlopen(url)
        content = response.read().decode('utf-8')
        response.close()
        o.write(content)
        if source_type == "js":
            o.write('</script>\n')
        else:
            o.write('</style>\n')
    else:
        raise ValueError("No file path in {l} for inline script.".format(l=line))


def locus_string(chr, start, end):
    if (end - start) == 1:
        return f'{chr}:{start + 1}'
    else:
        return f'{chr}:{start + 1}-{end}'


def main():
    parser = argparse.ArgumentParser()
    parser.add_argument("sites", help="vcf file defining variants, required")
    parser.add_argument("fasta", nargs="?", default=None, help="reference fasta file, required if --genome is not specified")
    parser.add_argument("--genome",  help="igv.js genome id (e.g. hg38)")
    parser.add_argument("--type", help="Report type.  Possible values are mutation and junctions.  Default is mutation")
    parser.add_argument("--ideogram", help="ideogram file in UCSC cytoIdeo format")
    parser.add_argument("--tracks", nargs="+", help="list of track files")
    parser.add_argument("--track-config", nargs="+", help="track json file")
    parser.add_argument("--sort",
                        help="initial sort option for alignment tracks.   Supported values include  BASE, STRAND, INSERT_SIZE, and MATE_CHR. Default value is BASE for single nucleotide variants, no sorting otherwise.  See the igv.js documentation for more information. ")
    parser.add_argument("--template", help="html template file", default=None)
    parser.add_argument("--output", help="output file name", default="igvjs_viewer.html")
    parser.add_argument("--info-columns", nargs="+", help="list of VCF info field names to include in variant table")
    parser.add_argument("--info-columns-prefixes", nargs="+",
                        help="list of prefixes of VCF info field names to include in variant table")
    parser.add_argument("--samples", nargs="+",
                        help="Space delimited list of sample (i.e. genotypes) names.  Used in conjunction with --sample-columns")
    parser.add_argument("--sample-columns", nargs="+",
                        help="list of VCF sample (genomtype) FORMAT field names to include in variant table")
    parser.add_argument("--flanking", help="genomic region to include either side of variant", default=1000)
    parser.add_argument("--standalone", help='Print more data', action='store_true')
    parser.add_argument("--title", help="optional title string")
    parser.add_argument("--sequence", help="Column of sequence (chromosome) name.  For tab-delimited sites file.",
                        default=None)
    parser.add_argument("--begin", help="Column of start position.  For tab-delimited sites file.", default=None)
    parser.add_argument("--end", help="column of end position. For tab-delimited sites file.", default=None)
    parser.add_argument("--zero_based",
                        help="Specify that the position in the data file is 0-based (e.g. UCSC files) rather than 1-based.",
                        default=None)
    parser.add_argument("--idlink", type=str, help="url link template for the VCF ID column")
    args = parser.parse_args()
    create_report(args)


if __name__ == "__main__":
    main()



from igv_reports import feature

def get_track_json_dict(filename):

    name = get_name(filename)
    format = feature.infer_format(filename)
    # Note: CRAM files are output in BAM format
    if format == 'cram':
        format = 'bam'
    # Note: BCF files are output in VCF format
    if format == 'bcf':
        format = 'vcf'
    type = get_track_type(format)
    return {
        "name": name,
        "url": filename,
        "type": type,
        "format": format
    }

def get_name(filename):

    idx = filename.rfind("/")
    if idx < 0:
        idx = filename.rfind("\\")
    period = filename.rfind(".")
    if idx < 0:
        if period < 0:
            return filename
        else:
            return filename[:period]
    else:
        idx += 1
        if period < 0:
            return filename[idx]
        else:
            return filename[idx:period]


def get_track_type(format):

    dict = {
        "bam": "alignment",
        "cram": "alignment",
        "bed": "annotation",
        "gff3": "annotation",
        "gff": "annotation",
        "gtf": "annotation",
        "bed": "annotation",
        "refgene": "annotation",
        "bcf": "variant",
        "vcf": "variant"
    }
    return dict[format]



from . import feature, bam, vcf

def getreader(config, filetype=None, args = None):

    path = config["url"]

    if not filetype:
        filetype = feature.infer_format(path)
    else:
        filetype = filetype.lower()

    if filetype == "bam" or filetype == "cram" :
        return bam.BamReader(filetype, path, args)

    elif filetype == "vcf" or filetype == "bcf":
        return vcf.VcfReader(path)

    else:
        return feature.get_featurereader(path)






import json
import sys
import html
from .feature import parse

class GenericTable:

    def __init__(self, file, info_columns = None, sequence = None, begin = None, end = None, zero_based=False):

        self.column_names = info_columns
        if sequence is not None and begin is not None and end is not None:
            seq_col = int(sequence) - 1
            start_col = int(begin) - 1
            end_col = int(end) - 1
        else:
            presets = self.determine_presets(file)
            seq_col = presets[0]
            start_col = presets[1]
            end_col = presets[2]
            start_offset = presets[3]


        if start_col < 0:
            msg = f"Invalid begin value: {begin}"
            sys.exit(msg)
        if end_col < 0:
            msg = f"Invalid end value: {end}"
            sys.exit(msg)

        self.features = []
        self.header = None
        self.rows = []

        rows = parse(file, 'tab')
        unique_id = 0
        start_offset = 0 if zero_based else 1
        for row in rows:

            #first line is the header
            if self.header is None:
                self.header = row
            else:
                self.rows.append(row)

                chr = row[seq_col]
                start = int(row[start_col]) - start_offset
                end = int(row[end_col])
                self.features.append((_Feature(chr, start, end), unique_id))
                unique_id += 1

    def to_JSON(self):


        obj = {
            "headers": ["unique_id"],
            "rows": []
        }

        if self.column_names is None:
            indeces = range(0, len(self.header))
        else:
            indeces = []
            for h in self.column_names:
                try:
                    idx = self.header.index(h)
                    indeces.append(idx)
                except:
                    print(f"Column {h} is not present")

        for i in indeces:
            obj["headers"].append(html.escape(self.header[i]))

        unique_id = 0
        for row in self.rows:
            r = [unique_id]
            for i in indeces:
                value = row[i] if i < len(row) else ""
                r.append(html.escape(value))
            obj["rows"].append(r)
            unique_id += 1

        return json.dumps(obj)

    def determine_presets(self, filename):

        filename = filename.lower()
        if (filename.endswith(".gz")):
            filename = filename[:-3]
        if filename.endswith(".bed"):
            return (0, 1, 2, True)
        elif filename.endswith(".gff") or filename.endswith("gff3") or filename.endswith("gtf"):
            return (0, 3, 4, False)
        elif filename.endswith(".maf"):
            return (4, 5, 6, False)
        else:
            msg = f"Unknown file format for: {filename} Please set --sequence, --start, and --end"
            sys.exit(msg)




class _Feature:

    def __init__(self, chr, start, end):
        self.chr = chr
        self.start = start
        self.end = end



import unittest
import pathlib
from igv_reports import varianttable, bedtable, generictable

class TableTest(unittest.TestCase):

    def test_get_header(self):

        vcf_path = str((pathlib.Path(__file__).parent / "data/minigenome/variants.vcf").resolve())
        table = varianttable.VariantTable(vcf_path)
        self.assertEqual(len(table.variants), 28)

    def test_bedtable(self):

        bed_file = str((pathlib.Path(__file__).parent / "data/minigenome/variants.bed").resolve())
        table = bedtable.BedTable(bed_file)
        json = table.to_JSON()
        self.assertEqual(len(table.features), 4)
        self.assertTrue(json)

    def test_bedpetable(self):

        bed_file = str((pathlib.Path(__file__).parent / "data/variants.bedpe").resolve())
        table = bedtable.BedpeTable(bed_file)
        json = table.to_JSON()
        self.assertEqual(len(table.features), 9)
        self.assertTrue(json)

    def test_gfftable(self):

        gff = str((pathlib.Path(__file__).parent / "data/minigenome/annotations.gtf.gz").resolve())
        table = bedtable.BedTable(gff)
        json = table.to_JSON()
        self.assertTrue(len(table.features), 2106)
        self.assertTrue(json)


    def test_maftable(self):

        maf_file = str((pathlib.Path(__file__).parent / "data/maf/tcga_test.maf").resolve())
        table = generictable.GenericTable(maf_file)

        json = table.to_JSON()
        self.assertEqual(len(table.features), 17)
        self.assertTrue(json)

    def test_maflite(self):

        maf_file = str((pathlib.Path(__file__).parent / "data/maf/test.maflite.tsv").resolve())

        info_columns = ["chr", "start", "end", "ref_allele", "alt_allele", "tumor_barcode"]
        sequence = 1
        start = 2
        end = 3

        table = generictable.GenericTable(maf_file, info_columns, sequence, start, end)

        json = table.to_JSON()
        self.assertEqual(29, len(table.features))
        self.assertTrue(json)


    def test_annovar(self):

        vcf_file = str((pathlib.Path(__file__).parent / "data/annotated_vcf/test.jannovar.vcf").resolve())

        #ANN=A|synonymous_variant|LOW|EGFR|1956|transcript|NM_001346897.1|Coding|19/26|c.2226G>A|p.(%3D)|2483/184056|2226/3276|742/1092||,A|synonymous_variant|LOW|EGFR|1956|transcript|NM_001346898.1|Coding|20/27|c.2361G>A|p.(%3D)|2618/184056|2361/3411|787/1137||,A|synonymous_variant|LOW|EGFR|1956|transcript|NM_001346899.1|Coding|19/27|c.2226G>A|p.(%3D)|2483/189060|2226/3498|742/1166||,A|synonymous_variant|LOW|EGFR|1956|transcript|NM_001346900.1|Coding|20/28|c.2202G>A|p.(%3D)|2415/98264|2202/3474|734/1158||,A|synonymous_variant|LOW|EGFR|1956|transcript|NM_001346941.1|Coding|14/22|c.1560G>A|p.(%3D)|1817/189060|1560/2832|520/944||,A|synonymous_variant|LOW|EGFR|1956|transcript|NM_005228.4|Coding|20/28|c.2361G>A|p.(%3D)|2618/189060|2361/3633|787/1211||;PROB_ABSENT=6086.16;PROB_ALT=0;PROB_ARTIFACT=3093.24;PROB_VERY_RARE=2789.3;SVLEN=.	DP:AF:OBS:SB	221:0.990543:116V-101V+2S-2N+:.

        table = varianttable.VariantTable(vcf_file, info_columns=["ANN"])
        json = table.to_JSON()
        self.assertTrue(json is not None)






from igv_reports.chralias import get_alias
from igv_reports.stream import getstream

class IdeogramReader:

    def __init__(self, file):

        self.aliastable = {}
        self.ideogram_map = {}

        lastchr = None
        result = ''

        f = None
        try:
            f = getstream(file)
            for line in f:
                tokens = line.split('\t')
                chr = tokens[0]

                if not chr == lastchr:

                    if lastchr is not None:
                        self.ideogram_map[lastchr] = result
                        self.aliastable[get_alias(lastchr)] = lastchr

                    lastchr = chr
                    result = ''
                else:
                    result += line
        finally:
            if f:
                f.close()

        # final chr
        if lastchr is not None:
            self.ideogram_map[lastchr] = result
            self.aliastable[get_alias(lastchr)] = lastchr


    def get_data(self, chr):

        if chr in self.aliastable:
            chrname = self.aliastable[chr]
        else:
            chrname = chr

        if chrname in self.ideogram_map:
            return self.ideogram_map[chrname]
        else:
            return ''
